package gui.schedulegui;

import java.util.Observable;
import java.util.Observer;
import java.util.HashMap;
import java.util.Map;
import java.awt.Color;
import java.awt.event.MouseListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.SwingConstants;
import javax.swing.BorderFactory;

import model.SchedulingModel;
import gui.GridPanel;

public abstract class Grid extends GridPanel implements Observer {
    protected Map<String, SlotLabel> labelMap, activeLabels;
    private SchedulingModel scheduleMdl;

    public Grid(int rows, int cols, SchedulingModel scheduleMdl, String title) {
        super(rows + 1, cols);
        setBorder(BorderFactory.createTitledBorder(title));

        this.scheduleMdl = scheduleMdl;
        scheduleMdl.addObserver(this);
        labelMap = new HashMap<>();
        activeLabels = new HashMap<>();
        
        for (int i = 1; i <= cols; i++) {
            add(new ColoredLabel(String.valueOf(i), Color.LIGHT_GRAY, SwingConstants.CENTER));
        }

        for (int row = 1; row <= rows; row++) {
            for (int col = 1; col <= cols; col++) {
                SlotLabel label = new SlotLabel();
                addListener(scheduleMdl, label); // Add mouse listener to this slot
                add(label);
                labelMap.put("(" + Integer.toString(row) + "," + Integer.toString(col) + ")", label);
            }
        }
    }

    /*
     * Adds MouseListener to each label
     */
    protected abstract void addListener(SchedulingModel scheduleMdl, SlotLabel s);

    /*
     * The action to perform in subclass when update happens
     */
    protected abstract void updateAction(SchedulingModel scheudleMdl);

    public void update(Observable obs, Object obj) { updateAction(scheduleMdl); }

    /*
     * Change activeLabels from subclass
     */
    protected void putActiveLabel(String id, SlotLabel s) { activeLabels.put(id, s); }
    protected void removeActiveLabel(String id) { activeLabels.remove(id); }

    /*
     * Return: string adress of slotlabel s
     */
    protected String getAdress(SlotLabel s) {
        for (String adress : labelMap.keySet()) {
            if(labelMap.get(adress).equals(s)) return adress;
        }
        return "";
    }

    /*
     * Parse task and slot to String id
     */
    protected String parse(int task, int slot) {
        return "(" + task + "," + slot + ")";
    }
    
    /*
     * Parse String to task
     */
    protected int parse(String id) {
        String[] str = id.replaceAll("\\D+"," ").trim().split(" "); //Extracts all numbers in string form
        return Integer.parseInt(str[0]);
    }

    /*
     * Clears all the active labels from the map
     */
    protected void clearActive() {
        for (Map.Entry<String, SlotLabel> entry : activeLabels.entrySet()) {
            int task = parse(entry.getKey());
            entry.getValue().toggle(task);
        }

        activeLabels.clear();
    }
}
