package gui.tasksetgui;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.Color;
import javax.swing.JTextField;

public class EmptyLabel extends JTextField implements FocusListener {

    private String previousText;
    private EmptyListener emptyListener;

    public EmptyLabel(String text) {
        // Front-end Characteristics
        super(text);
        setBackground(Color.WHITE);
        setOpaque(true);
        setDocument(new TaskLabelFilter());
        setText(text);

        // Add Listener for when focus is placed inside
        addFocusListener(this);

        // Add Listener that does something with object when <Enter> is pressed
        addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getText().matches("[0-9]+")) {
                    emptyListener.emptyUpdate();
                }
            }
        });
    }

    /*
     * Sets observer of this class 
     * (Would have used observable superclass unless I already had a superclass)
     */
    public void addEmptyListener(EmptyListener emptyListener) {
        this.emptyListener = emptyListener;
    }

    /*
     * FocusListener Functions.
     */

    /*
     * Invoked when a component gains the keyboard focus.
     */
    public void focusGained(FocusEvent e) {
        previousText = getText();
    }

    /*
     * Invoked when a component loses the keyboard focus.
     */
    public void focusLost(FocusEvent e) {
        if (!previousText.equals(getText()) && getText().matches("[0-9]+")) {
            emptyListener.emptyUpdate();
        } else {
            setText(previousText);
        }
    }
}
