package gui.tasksetgui;

import java.util.HashMap;
import java.util.Map;
import java.awt.Color;

import gui.GridPanel;

public class EmptyTaskGrid extends GridPanel implements EmptyListener {

    private String[] attributes = new String[] {"Pri", "e", "p", "d"};
    private boolean full = false;
    private Map<String, EmptyLabel> labelMap;
    private EmptyListener emptyListener;

    // attributes should contain [prio, exectime, period, deadline]
    public EmptyTaskGrid(String prio) {
        super(1, 4);
        labelMap = new HashMap<>();
        // Add priority label
        EmptyLabel prioLabel = new EmptyLabel(prio);
        prioLabel.addEmptyListener(this);
        prioLabel.setEditable(false); // Forbids people from changing priority
        add(prioLabel);
        labelMap.put(attributes[0], prioLabel);

        
        for (int i = 1; i < 4; i++) {
            EmptyLabel label = new EmptyLabel("");
            label.addEmptyListener(this);

            add(label);
            labelMap.put(attributes[i], label);
        }

        colorize(Color.RED);
    }
    
    /*
     * return: attributes from this gridLine
     */
    public int[] getAttributes() {
        int[] attr = new int[4];
        for (int i = 0; i < 4; i++) {
            EmptyLabel label = labelMap.get(attributes[i]);
            attr[i] = Integer.parseInt(label.getText());
        }
        return attr;
    }

    /*
     * Check whether this taskgrid is still empty
     */
    public boolean isEmpty() {
        for (int i = 1; i < 4; i++) {
            EmptyLabel label = labelMap.get(attributes[i]);
            if (label.getText().isEmpty()) {
                return true;
            }
        }
        return false;
    }

    /*
     * The function to be performed when the observable class is updated.
     * NOTE: TWO INTERFACES ARE NEEDED SUCH THAT THE UI ISN'T UPDATED ONLY WHEN MARKED.
     */
    public void emptyUpdate() {
        emptyListener.emptyUpdate();
    }

    /*
     * Sets observer of this class 
     * (Would have used observable superclass unless I already had a superclass)
     */
    public void addEmptyListener(EmptyListener emptyListener) {
        this.emptyListener = emptyListener;
    }

    //Private help function to reduce code size
    private void colorize(Color c) {
        for (EmptyLabel tl : labelMap.values()) {
            tl.setBackground(c);
        }
    }
}
