package gui.tasksetgui;

import javax.swing.JTextField;
import javax.swing.Box;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import java.util.Optional;

public abstract class MultiInputPane extends JPanel {
    private JTextField[] fields;

    public MultiInputPane(int nbrFields) {
        String[] labels = labelTitles();
        fields = new JTextField[nbrFields];

        // Create text fields and labels.
        for (int i = 0; i < nbrFields; i++) {
            fields[i] = new JTextField(5);
            add(new JLabel(labels[i]));
            add(fields[i]);
            if (i < nbrFields - 1) {
                add(Box.createHorizontalStrut(15));
            }
        }
    }

    /*
     * Generate label titles in array from subclass
     */
    protected abstract String[] labelTitles();

    /*
     * generate dialog message
     */
    protected abstract String generateDialogMessage();

    /*
     * return Optional int[] or empty optional if cancel was pressed.
     */
    public Optional<int[]> generateDialog() throws NumberFormatException {
        int result = JOptionPane.showConfirmDialog(null, this,
               generateDialogMessage(), JOptionPane.OK_CANCEL_OPTION);

        int[] values = new int[fields.length];
        if (result == JOptionPane.OK_OPTION) {
            for (int i = 0; i < fields.length; i++) {
                try {
                    values[i] = Integer.parseInt(fields[i].getText().trim());
                } catch (NumberFormatException e) {
                    throw e;
                }
            }
        } else {
            return Optional.empty();
        }

        return Optional.of(values);
    }
}

