package gui.tasksetgui;

import java.util.HashMap;
import java.util.Map;
import java.awt.Color;

import gui.GridPanel;
import gui.UpdateListener;

public class TaskGrid extends GridPanel implements UpdateListener, MarkListener {

    private String[] attributes = new String[] {"Pri", "e", "p", "d"};
    private Map<String, TaskLabel> labelMap;
    private UpdateListener updateListener;
    private MarkListener markListener;

    // attributes should contain [prio, exectime, period, deadline]
    public TaskGrid(String[] taskAttributes) {
        super(1, 4);
        labelMap = new HashMap<>();
        
        for (int i = 0; i < 4; i++) {
            TaskLabel label = new TaskLabel(taskAttributes[i]);
            label.addUpdateListener(this);
            label.addMarkListener(this);

            if (i == 0) label.setEditable(false); // Forbids people from changing priority

            add(label);
            labelMap.put(attributes[i], label);
        }
    }
    
    public TaskGrid(String prio, String exec, String period, String deadline) {
        this(new String[] {prio, exec, period, deadline});
    }

    /*
     * return: attributes from this gridLine
     */
    public int[] getAttributes() {
        int[] attr = new int[4];
        for (int i = 0; i < 4; i++) {
            TaskLabel label = labelMap.get(attributes[i]);
            attr[i] = Integer.parseInt(label.getText());
        }
        return attr;
    }

    /*
     * Compares two taskgrid objects and returns the equality between the two
     */
    public boolean equals(TaskGrid tg) {
        return getAttributes()[0] == tg.getAttributes()[0];
    }

    /*
     * The function to be performed when the observable class is updated.
     * NOTE: TWO INTERFACES ARE NEEDED SUCH THAT THE UI ISN'T UPDATED ONLY WHEN MARKED.
     */
    public void update(Object obj) {
        updateListener.update(this);
    }

    /*
     * The function to be performed when the tasklabel class is marked.
     * NOTE: TWO INTERFACES ARE NEEDED SUCH THAT THE UI ISN'T UPDATED ONLY WHEN MARKED.
     */
    public void markUpdate(Object obj) {
        markListener.markUpdate(this);
    }

    /*
     * Sets observer of this class 
     * (Would have used observable superclass unless I already had a superclass)
     */
    public void addUpdateListener(UpdateListener updateListener) {
        this.updateListener = updateListener;
    }

    /*
     * Sets observer of this class 
     * (Would have used observable superclass unless I already had a superclass)
     */
    public void addMarkListener(MarkListener markListener) {
        this.markListener = markListener;
    }

    /*
     * Marks all the tasklabels of this taskgrid.
     */
    public void mark() {
        colorize(Color.YELLOW);
    }

    /*
     * Unmarks all the tasklabels of this taskgrid.
     */
    public void unmark() {
        colorize(Color.WHITE);
    }

    //Private help function to reduce code size
    private void colorize(Color c) {
        for (TaskLabel tl : labelMap.values()) {
            tl.setBackground(c);
        }
    }
}
