package gui.tasksetgui;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.Color;
import javax.swing.JTextField;

import gui.UpdateListener;

public class TaskLabel extends JTextField implements FocusListener {

    private String previousText;
    private UpdateListener updateListener;
    private MarkListener markListener;

    public TaskLabel(String text) {
        // Front-end Characteristics
        super(text);
        setBackground(Color.WHITE);
        setOpaque(true);
        setDocument(new TaskLabelFilter());
        setText(text);

        // Add Listener for when focus is placed inside
        addFocusListener(this);

        // Add Listener that does something with object when <Enter> is pressed
        addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (getText().matches("[0-9]+")) {
                    updateListener.update();
                }
            }
        });

        // Add Listener that does something When object is clicked
        addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                markListener.markUpdate(this);
            }
        });
    }

    /*
     * Sets observer of this class 
     * (Would have used observable superclass unless I already had a superclass)
     */
    public void addUpdateListener(UpdateListener updateListener) {
        this.updateListener = updateListener;
    }

    /*
     * Sets observer of this class 
     * (Would have used observable superclass unless I already had a superclass)
     */
    public void addMarkListener(MarkListener markListener) {
        this.markListener = markListener;
    }
    
    /*
     * FocusListener Functions.
     */

    /*
     * Invoked when a component gains the keyboard focus.
     */
    public void focusGained(FocusEvent e) {
        previousText = getText();
    }

    /*
     * Invoked when a component loses the keyboard focus.
     */
    public void focusLost(FocusEvent e) {
        if (!previousText.equals(getText()) && getText().matches("[0-9]+")) {
            updateListener.update();
        } else {
            setText(previousText);
        }
    }
}
