package model;

import tasks.Taskset;
import tasks.Task;
import strategy.SchedulingStrategy;
import strategy.NoneStrategy;

import java.util.Arrays;

public class Schedule {
    private SchedulingStrategy strategy;
    private int[] schedule;
    private Taskset ts;

    public Schedule(Taskset ts) {
        this.ts = ts;
        strategy = new NoneStrategy();
    }

    public void setStrategy(SchedulingStrategy strategy) {
        this.strategy = strategy;
        schedule = strategy.generate(ts);
    }

    public SchedulingStrategy getStrategy() { return strategy; }

    public boolean isValid() {
        // If strategy does not exist or utilization is greater than 1:
        // return false
        if (strategy instanceof NoneStrategy || 
            ts.utilization() > 1) 
            return false;

        for (int taskNbr : ts.getPriorities()) {
            Task t = ts.getTask(taskNbr).get(); //I know it exists so I can collect it from the optional directly

            // Introduce this check to prevent deadline from being longer than period.
            if (t.deadline() > t.period()) return false;

            //Loops through the length of the hyperperiod as long as the generated schedule has enough length.
            for (int i = 0; i < ts.hyperperiod(); i += t.period()) { 
                int added = 0;
                for (int j = i; j < i + t.deadline(); j++) {
                    if (schedule[j] == taskNbr) added++;
                }
                if (added != t.exectime()) return false;
            }
        } 
        return true;
    }

    public int[] generateSchedule() {
        schedule = strategy.generate(ts);
        return schedule;
    }
}
