package strategy;

import tasks.Taskset;
import tasks.Task;

import java.util.PriorityQueue;
import java.util.Queue;
import java.util.Arrays;

public abstract class QueueStrategy implements SchedulingStrategy {

    /*
     * Generate schedule from taskset ts based on the desired strategy
     * Return: int array containing task number i at slot position j. Empty array if taskset is invalid.
     */
    public int[] generate(Taskset ts) {
        // If we haven't added any tasks yet: return empty array
        if (ts.nbrTasks() == 0) return new int[0]; 

        int[] schedule = new int[ts.hyperperiod()];
        Arrays.fill(schedule, 0);
        Queue<Integer> taskQ = new PriorityQueue<>();

        for (int t = 0; t < schedule.length; t++) {
            taskQ = updateQueue(taskQ, ts, t);
            if (taskQ.size() != 0) schedule[t] = taskQ.poll();
        }

        return schedule;
    }

    /*
     * Return: Current version of the ready queue
     */
    protected abstract Queue<Integer> updateQueue(Queue<Integer> q, Taskset ts, int time);
}
